
import psycopg2
from psycopg2.extras import DictCursor as _CURSOR_FACTORY

# Importing db connection configuration
from conf import\
    DB_HOST,DB_NAME,DB_USER,DB_PASSWD

from speaker import *
from eccez import *
#
##
### EXPORTING INTERFACE
##
#

_DBDA_ATTRS=['commit',      \
             'mogrify',     \
             'ov_query',    \
             'rollback',    \
             'oc_query',    \
             'or_query',    \
             'mr_query',    \
             'beginTrans',  \
             'lastrowid']

def dbda_iface_init(c_to,c_from):
    for a in _DBDA_ATTRS:
        na= getattr(c_from, a, lambda: None)
        setattr(c_to, a, na)

def _connectStr():
    return DB_HOST and\
        "dbname=%s user=%s host=%s password=%s"\
            % (DB_NAME,DB_USER,DB_HOST,DB_PASSWD) or\
        "dbname=%s user=%s password=%s"\
            % (DB_NAME,DB_USER,DB_PASSWD) 
#
##
### The DbDa ClaSs
##
#

class DB:
    def __init__(self):
        self.connection= None
        self.cursor= None

    def _get_connection(self):
        return self.connection

    def _get_cursor(self):
        return self.cursor

    def connect(self, connection= None):
        if connection is None and self.connection is None:
            self.connection= psycopg2.connect(_connectStr())
            self.cursor= self.connection.cursor(\
                    cursor_factory=_CURSOR_FACTORY)
            self.connected()
        else:
            self.connection= connection
            self.cursor=self.connection.getcursor()

    def connected(self):
        self.get_connection= self._get_connection
        self.get_cursor= self._get_cursor

    def get_connection(self):
        self.connect()
        return self._get_connection()

    def get_cursor(self):
        self.connect()
        return self._get_cursor()

MEDIUM= DB()

def dbda_init(db_conn=None):
    MEDIUM.connect(db_conn)

class DbDa(object):

    exporting_methods= [ 'commit',      \
                         'mogrify',     \
                         'ov_query',    \
                         'rollback',    \
                         'oc_query',    \
                         'or_query',    \
                         'mr_query',    \
                         'beginTrans',  \
                         'lastrowid' ]

    @staticmethod
    def _ready():
        return getattr(DbDa, '_the_instance', None)

    def __new__(cls, *p, **k):
        i= DbDa._ready()
        if i is None:
            DbDa._the_instance= object.__new__(DbDa)
            if cls is DbDa:
                return DbDa._the_instance
            DbDa.__init__(DbDa._the_instance)
            i= DbDa._the_instance

        o= object.__new__(cls)

        for attr_name in DbDa.exporting_methods:
            setattr(o, attr_name, getattr(i, attr_name))

        return o

    def __init__(self, logger= None):

        self.transaction=0
        if logger is not None:
            speaker_iface_init(self, logger)
        
    def _extremes(self):
        return MEDIUM.get_cursor(), MEDIUM.get_connection()

#
##
### INTERNALS
##
#

    def _get_query(self,s):
        self.log_debug("QSQL- %s",s,DBG_NORMAL)
        cursor, dbda= self._extremes()
        try:
            cursor.execute(s)
            res= cursor.fetchall()
        except Exception,inst:
            dbda.rollback()
            s="Unable to execute query: %s" % str(inst)
            self.log_raise(s,QueryError(s))
        return res

    def _quiet_query(self,s):
        self.log_debug("QSQL- %s",s,DBG_NORMAL)
        cursor, dbda= self._extremes()
        try:
            cursor.execute(s)
            if not self.transaction:
                dbda.commit()
            else:
                self.log_debug("Query not committed.",DBG_INSANE)
        except Exception,inst:
            dbda.rollback()
            s="Unable to execute query: %s" % str(inst)
            self.log_raise(s,QueryError(s))

    def _setTrans(self):
        self.transaction+=1
        self.log_debug("Incremented transaction: level=%s.",\
                YELLOW(self.transaction),DBG_INSANE)

    def _unsetTrans(self):
        self.transaction-=1
        self.log_debug("Decremented transaction: level=%s.",\
                YELLOW(self.transaction),DBG_INSANE)

    def _resetTrans(self):
        self.log_debug("Resetting transaction at level 0.",DBG_INSANE)
        self.transaction=0
#
##
### PUB
##
#
    def beginTrans(self):
        if self.transaction:
            self.log_debug("Nested transaction, level=%s.",\
                    YELLOW(self.transaction),DBG_SOFT)
        self.transaction+=1
        self.log_debug("Incremented transaction: level=%s.",\
                YELLOW(self.transaction),DBG_INSANE)
        return

    def commit(self):
        if not self.transaction:
            self.log_int_raise("What do you want to commit? "\
                    "There is no active transaction.")
        self._unsetTrans()
        if self.transaction:
            self.log_debug("Transaction is not at level 0, not committing.",\
                    DBG_SOFT)
        else:
            dbda= MEDIUM.get_connection()
            dbda.commit()
	    self.log_debug("Queries committed.",DBG_SOFT)

    def rollback(self):
        if not self.transaction:
            self.log_debug("There is no active transaction. Maybe "\
                    "rollback was called nested?",DBG_SOFT)
        else:
            dbda= MEDIUM.get_connection()
            self._resetTrans()
            dbda.rollback()
            self.log_debug("Rollbacked changes.",DBG_SOFT)

    def mogrify(self,s,d):
        """ Mix a string and a dict: see cursor.mogrify."""
        cursor= MEDIUM.get_cursor()
        s= cursor.mogrify(s,d)
        return s

    def lastrowid(self):
        cursor= MEDIUM.get_cursor()
        return cursor.lastrowid

    def oc_query(self,s):
        """ One Commit Query: no returns."""
        self._quiet_query(s)

    def ov_query(self,s):
        """ One Value Query: returns one only value."""
        res=self._get_query(s)
        try:
            return res[0][0]
        except:
            self.log_int_raise(
                "No value for the executed OV-query.")

    def or_query(self,s):
        """ One Record Query: returns a dict."""
        res=self._get_query(s)
        try:
            res=res[0]
        except:
            self.log_int_raise(
               "No values for the query.")
        d={}
        for k in res.keys():
            d[k]=res[k]
        return d

    def mr_query(self,s):
        """ Multiple Records Query: return a dict list."""
        res=self._get_query(s)
        dict_list=[]
        for r in res:
            d={}
            for key in r.keys():
                d[key]=r[key]
            dict_list.append(d)
        return dict_list
