import traceback

# Module symbols are here.
from speaker_h import *
    
#
##
### The Speaker Heart
##
#

#
##
### EXPORTING INTERFACE
##
#
## Speaker attributes:
#
_SPEAKER_ATTRS=[\
                'log_info',\
                'log_error',\
                'log_raise',\
                'log_debug',\
                'log_level',\
                'log_int_raise',\
                'set_log_level',\
                'set_log_to_stdout']

def speaker_iface_init(c_to,c_from):
    for a in _SPEAKER_ATTRS:
        na= getattr(c_from, a, lambda: None)
        setattr(c_to, a, na)



#
##
### The SpeAKeR Class
##
#

class Speaker:
            
    def __init__(self, prefix   = LOG_PREFIX,\
                 log_level      = LOG_LEVEL,\
                 log_to_stdout  = LOG_TO_STDOUT,\
                 log_to_syslog  = LOG_TO_SYSLOG,\
                 log_trback     = LOG_TRBACK,\
                 log_facility   = LOG_FACILITY,\
                 log_opts       = LOG_OPTS,\
                 log_color      = LOG_COLOR):

        """ Init Me, give me Life """

        self.log_prefix         = prefix 
        self.log_level          = log_level
        self.log_to_stdout      = log_to_stdout
        self.log_to_syslog      = log_to_syslog
        self.log_trback         = log_trback
        self.log_facility       = log_facility
        self.log_opts           = log_opts
        self.log_color          = log_color

        # Syslog init...
        if self.log_to_syslog:
            self._syslog_connect()

        global LOG_COLOR
        LOG_COLOR= log_color

    def _raise(self,ex):
        """ Internal. """
        raise ex

    def _outlog(self,s):

        log= "%s%s" % (self._tb(), s)

        if self.log_to_stdout:
            print log

        if self.log_to_syslog:
            try:
                syslog(log)
            except:
                pass

    def _syslog_connect(self):
        """ Internal. """
        try:
            openlog(self.log_prefix, self.log_opts, self.log_facility)
        except Exception, s:
            self.log_int_raise("Speaker: can not connect to syslog: %s", s)

    def _tb(self):
        if not self.log_trback:
            return ""
        order=4
        l=traceback.extract_stack(limit=order)
        fil=l[0][0]
        fil=fil.split("/")[-1]
        line=l[0][1]
        f=l[0][2]
        arrow=PURPLE('->')
        return "%(fil)s:%(line)d:%(f)s() %(arrow)s " % locals()

#
##
### Public
##
#

    def set_log_to_stdout(self,arg):
        self.log_to_stdout= arg

    def set_log_level(self,lvl):
        """ Set the debug level."""
        self.log_level= lvl

    def log_debug(*args):
        """ log_debug(*args): --> logging
        
        args[0] is self, args[-1] is the debug level.
        args[1:-1] is the un-formatted string.
        """ 
        me=args[0]
        if args[-1]>me.log_level:
            return
        s=(args[1] % tuple(args[2:-1]))
        me._outlog(LOG_DBG_SYMBOL+s) 

    def log_info(*args):
        """ log_info(*args): --> logging
        
        args[0] is self.
        args[1:] is the un-formatted string.
        """ 
        me=args[0]
        s=(args[1] % tuple(args[2:]))
        me._outlog(LOG_INF_SYMBOL+s) 

    def log_error(*args):
        """ log_error(*args): --> logging
        
        args[0] is self.
        args[1:] is the un-formatted string.
        """ 
        s=(args[1] % tuple(args[2:]))
        args[0]._outlog("%s%s"% (LOG_ERR_SYMBOL,s)) 

    def log_raise(*args):
        """ log_raise(*args): --> logging and Raising
        
        args[0] is self.
        args[1:-1] is the un-formatted string.
        args[-1] is the Exception to be raised.
        """ 
        me= args[0]
        exc= args[-1]
        s= (args[1] % tuple(args[2:-1]))
        me._outlog("%s%s"% (LOG_ERR_SYMBOL,s))
        me._raise(exc)

    def log_int_raise(*args):
        """ log_raise(*args): --> logging and Raising InternalError
        
        args[0] is self.
        args[1:-1] is the un-formatted string.
        """ 
        me= args[0]
        s=args[1] % tuple(args[2:])
        me._outlog("%s%s"% (LOG_ERR_SYMBOL,s))
        me._raise(InternalError(s))



#
##
### Colors
####
#####

def RED(s):
    return LOG_COLOR and\
        "%s%s%s" %(REDCOL,s,DEFCOL) or s
def GREEN(s):
    return LOG_COLOR and\
        "%s%s%s" %(GREENCOL,s,DEFCOL) or s
def YELLOW(s):
    return LOG_COLOR and\
        "%s%s%s" %(YELLOWCOL,s,DEFCOL) or s
def PURPLE(s):
    return LOG_COLOR and\
        "%s%s%s" %(PURPLECOL,s,DEFCOL) or s

